#include <afxwin.h>
#include <afxext.h>
#include <afxcoll.h>
#include <stdio.h>
#include <string.h>
#include "resource.h"

/* --- Rubber Band Effect Variables --- */
int choice=1;
static CPoint startp,prevp,currp;
BOOL ftime=FALSE,lbdown=FALSE;


/* GDI Objects  variables*/
COLORREF White = RGB(255,255,255);
COLORREF PenColor = RGB(0,0,0);
COLORREF BrushColor = RGB(255,255,255);

/* ----- Class derived from CFrameWnd class ----- */
class MyWindow:public CFrameWnd
{
public:
	/* Generalized Functions*/
	void drawshape(COLORREF pen,COLORREF brush, CPoint p1,CPoint p2);
	COLORREF GetColor();

	/* Constructor */
	MyWindow();
	
	/* --- Function prototyping for ON_COMMAND macros --- */
	void OnQuit();		
	void OnFont();

	void OnLine();		
	void OnRectangle();
	void OnEllipse();		

	void OnPenRed();			
	void OnPenGreen();		
	void OnPenBlue();
	void OnPenColor();

	void OnBrushRed();
	void OnBrushGreen();
	void OnBrushBlue();
	void OnBrushColor();

	
	

	/* -- Protyping for System defined functions -- */
	void OnMouseMove(UINT,CPoint);
	void OnLButtonDown(UINT,CPoint);
	void OnLButtonUp(UINT,CPoint);
	void OnChar(UINT nChar,UINT nRep,UINT Flags);

	DECLARE_MESSAGE_MAP()
};
/* ----- MESSAGE MAP ENTRIES ----- */
BEGIN_MESSAGE_MAP(MyWindow,CFrameWnd)
	/* USER DEFINED */
	
	ON_COMMAND(ID_LINE,OnLine)
	ON_COMMAND(ID_RECTANGLE,OnRectangle)
	ON_COMMAND(ID_ELLIPSE,OnEllipse)

	ON_COMMAND(ID_PEN_RED,OnPenRed)
	ON_COMMAND(ID_PEN_BLUE,OnPenBlue)
	ON_COMMAND(ID_PEN_GREEN,OnPenGreen)
	ON_COMMAND(ID_PEN_COLOR,OnPenColor)
	
	ON_COMMAND(ID_BRUSH_RED,OnBrushRed)
	ON_COMMAND(ID_BRUSH_BLUE,OnBrushBlue)
	ON_COMMAND(ID_BRUSH_GREEN,OnBrushGreen)
	ON_COMMAND(ID_BRUSH_COLOR,OnBrushColor)

	ON_COMMAND(ID_FONT,OnFont)
	ON_COMMAND(ID_QUIT,OnQuit)

	/* SYSTEM DEFINED */
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_CHAR()
END_MESSAGE_MAP()

/* ----------------------------------------------------------*/
/* ----------------------------------------------------------*/
/* ----------------------------------------------------------*/

MyWindow::MyWindow()
{
	Create(NULL,"GDI Objects",
				WS_OVERLAPPEDWINDOW,rectDefault,
				NULL,MAKEINTRESOURCE(IDR_MENU1),NULL);
}

void MyWindow::drawshape(COLORREF pen,COLORREF brush, CPoint p1,CPoint p2)
{
	CPen	ColorPen;
	CBrush	ColorBrush;

	ColorPen.CreatePen(PS_SOLID,5,pen);
	ColorBrush.CreateSolidBrush(brush);

	CClientDC MyDC(this);
	MyDC.SelectObject(ColorPen);
	MyDC.SelectObject(ColorBrush);

	switch(choice)
	{
	case 1:
		MyDC.MoveTo(p1.x,p1.y);
		MyDC.LineTo(p2.x,p2.y);
		break;
	case 2:
		MyDC.Rectangle(p1.x,p1.y,p2.x,p2.y);
		break;

	case 3:
		MyDC.Ellipse(p1.x,p1.y,p2.x,p2.y);
		break;
	}
}

COLORREF MyWindow::GetColor()
{
	CColorDialog colorbox;

	if(colorbox.DoModal()==IDOK)
	{
		COLORREF color;
		color=colorbox.GetColor();
		return color;
	}
	return NULL;
}


void MyWindow::OnLine()      {choice=1;}
void MyWindow::OnEllipse()   {choice=3;}
void MyWindow::OnRectangle() {choice=2;}

void MyWindow::OnPenRed()	{PenColor=RGB(255,0,0);}
void MyWindow::OnPenGreen() {PenColor=RGB(0,255,0);}
void MyWindow::OnPenBlue()	{PenColor=RGB(0,0,255);}
void MyWindow::OnPenColor() {PenColor=GetColor();}

void MyWindow::OnBrushRed()		{BrushColor=RGB(255,0,0);}
void MyWindow::OnBrushGreen()	{BrushColor=RGB(0,255,0);}
void MyWindow::OnBrushBlue()	{BrushColor=RGB(0,0,255);}
void MyWindow::OnBrushColor()	{BrushColor=GetColor();}


void MyWindow::OnFont()
{

	LOGFONT lf;
	CFontDialog fd(&lf);

	if(fd.DoModal()==IDOK)
	{
		CClientDC dc(this);
		CFont font;
		font.CreateFontIndirect(&lf);
		dc.SelectObject(&font);
		dc.SetTextColor(fd.GetColor());
		dc.TextOut(0,50,"abcdefghijklmnopqrstuvwxyz",26);
	}
	
}

void MyWindow::OnChar(UINT nChar,UINT nRep,UINT Flags)
{
	char str[5];
	sprintf(str,"%c",nChar);
	CClientDC TextDC(this);
	TextDC.TextOut(0,300,"   ");
	TextDC.TextOut(0,300,str);
}


/* RUBBERBAND EFFECT */
void MyWindow::OnMouseMove(UINT u,CPoint cp)
{
	/* ----- Displaying mouse co-ordinates on the status bar ----- */
	int x=cp.x;int y=cp.y;
	
	/* ----- RUBBER BAND EFFECT IMPLEMENTATION ----- */
	if(!lbdown)
		return;

	if(ftime)
	{
		currp=cp;
		ftime=FALSE;
		return;
	}

	prevp=currp;
	currp=cp;
	
	drawshape(White,White,startp,prevp);
	drawshape(PenColor,BrushColor,startp,currp);
}


void MyWindow::OnLButtonDown(UINT u,CPoint sp)
{
	ftime=TRUE;
	lbdown=TRUE;
	startp=sp;
}

void MyWindow::OnLButtonUp(UINT u,CPoint lp)
{
	currp=lp;
	drawshape(PenColor,BrushColor,startp,currp);
	lbdown=FALSE;
}

void MyWindow::OnQuit() {PostQuitMessage(0);}

/* ----------------------------------------------------------*/
/* ----------------------------------------------------------*/
/* ----------------------------------------------------------*/



class MyApp:public CWinApp
{
public:
	BOOL InitInstance()
	{
		MyWindow *MyWindowObject;
		MyWindowObject=new MyWindow;
		MyWindowObject->UpdateWindow();
		MyWindowObject->ShowWindow(3);
		m_pMainWnd=MyWindowObject; 
		return TRUE;
	}
};
MyApp TheApp;
